@@ -5,7 +5,7 @@
  * object level logging, and fully-qualified object names for all DML and DDL
  * statements where possible (See README.md for details).
  *
- * Copyright (c) 2014-2021, PostgreSQL Global Development Group
+ * Copyright (c) 2014-2017, PostgreSQL Global Development Group
  *------------------------------------------------------------------------------
  */
 #include "postgres.h"
@@ -25,7 +25,6 @@
 #include "miscadmin.h"
 #include "libpq/auth.h"
 #include "nodes/nodes.h"
-#include "nodes/params.h"
 #include "tcop/utility.h"
 #include "tcop/deparse_utility.h"
 #include "utils/acl.h"
@@ -36,6 +35,7 @@
 #include "utils/rel.h"
 #include "utils/syscache.h"
 #include "utils/timestamp.h"
+#include "utils/varlena.h"
 
 PG_MODULE_MAGIC;
 
@@ -599,6 +599,7 @@ log_audit_event(AuditEventStackItem *stackItem)
             {
                 /* READ statements */
                 case T_CopyStmt:
+                case T_DeclareCursorStmt:
                 case T_SelectStmt:
                 case T_PrepareStmt:
                 case T_PlannedStmt:
@@ -970,7 +971,7 @@ log_select_dml(Oid auditOid, List *rangeTabls)
     foreach(lr, rangeTabls)
     {
         Oid relOid;
-        Oid relNamespaceOid;
+        Relation rel;
         RangeTblEntry *rte = lfirst(lr);
 
         /* We only care about tables, and can ignore subqueries etc. */
@@ -993,10 +994,13 @@ log_select_dml(Oid auditOid, List *rangeTabls)
          * false) then filter out any system relations here.
          */
         relOid = rte->relid;
-        relNamespaceOid = get_rel_namespace(relOid);
+        rel = relation_open(relOid, NoLock);
 
-        if (!auditLogCatalog && IsSystemNamespace(relNamespaceOid))
+        if (!auditLogCatalog && IsSystemNamespace(RelationGetNamespace(rel)))
+        {
+            relation_close(rel, NoLock);
             continue;
+        }
 
         /*
          * Default is that this was not through a grant, to support session
@@ -1093,8 +1097,10 @@ log_select_dml(Oid auditOid, List *rangeTabls)
 
         /* Get a copy of the relation name and assign it to object name */
         auditEventStack->auditEvent.objectName =
-            quote_qualified_identifier(
-                get_namespace_name(relNamespaceOid), get_rel_name(relOid));
+            quote_qualified_identifier(get_namespace_name(
+                                           RelationGetNamespace(rel)),
+                                       RelationGetRelationName(rel));
+        relation_close(rel, NoLock);
 
         /* Perform object auditing only if the audit role is valid */
         if (auditOid != InvalidOid)
@@ -1289,7 +1295,7 @@ pgaudit_ExecutorStart_hook(QueryDesc *queryDesc, int eflags)
 
         /* Initialize the audit event */
         stackItem->auditEvent.commandText = queryDesc->sourceText;
-        stackItem->auditEvent.paramList = copyParamList(queryDesc->params);
+        stackItem->auditEvent.paramList = queryDesc->params;
     }
 
     /* Call the previous hook or standard function */
@@ -1338,10 +1344,11 @@ pgaudit_ExecutorCheckPerms_hook(List *rangeTabls, bool abort)
  * Hook ProcessUtility to do session auditing for DDL and utility commands.
  */
 static void
-pgaudit_ProcessUtility_hook(Node *parsetree,
+pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
                             const char *queryString,
                             ProcessUtilityContext context,
                             ParamListInfo params,
+                            QueryEnvironment *queryEnv,
                             DestReceiver *dest,
                             char *completionTag)
 {
@@ -1357,38 +1364,19 @@ pgaudit_ProcessUtility_hook(Node *parsetree,
         /* Process top level utility statement */
         if (context == PROCESS_UTILITY_TOPLEVEL)
         {
-            /*
-             * If the stack is not empty then the only allowed entries are open
-             * select, show, and explain cursors
-             */
             if (auditEventStack != NULL)
-            {
-                AuditEventStackItem *nextItem = auditEventStack;
-
-                do
-                {
-                    if (nextItem->auditEvent.commandTag != T_SelectStmt &&
-                        nextItem->auditEvent.commandTag != T_VariableShowStmt &&
-                        nextItem->auditEvent.commandTag != T_ExplainStmt)
-                    {
-                        elog(ERROR, "pgaudit stack is not empty");
-                    }
-
-                    nextItem = nextItem->next;
-                }
-                while (nextItem != NULL);
-            }
+                elog(ERROR, "pgaudit stack is not empty");
 
             stackItem = stack_push();
-            stackItem->auditEvent.paramList = copyParamList(params);
+            stackItem->auditEvent.paramList = params;
         }
         else
             stackItem = stack_push();
 
         stackId = stackItem->stackId;
-        stackItem->auditEvent.logStmtLevel = GetCommandLogLevel(parsetree);
-        stackItem->auditEvent.commandTag = nodeTag(parsetree);
-        stackItem->auditEvent.command = CreateCommandTag(parsetree);
+        stackItem->auditEvent.logStmtLevel = GetCommandLogLevel(pstmt->utilityStmt);
+        stackItem->auditEvent.commandTag = nodeTag(pstmt->utilityStmt);
+        stackItem->auditEvent.command = CreateCommandTag(pstmt->utilityStmt);
         stackItem->auditEvent.commandText = queryString;
 
         /*
@@ -1399,28 +1387,15 @@ pgaudit_ProcessUtility_hook(Node *parsetree,
             stackItem->auditEvent.commandTag == T_DoStmt &&
             !IsAbortedTransactionBlockState())
             log_audit_event(stackItem);
-
-        /*
-         * A close will free the open cursor which will also free the close
-         * audit entry. Immediately log the close and set stackItem to NULL so
-         * it won't be logged later.
-         */
-        if (stackItem->auditEvent.commandTag == T_ClosePortalStmt)
-        {
-            if (auditLogBitmap & LOG_MISC && !IsAbortedTransactionBlockState())
-                log_audit_event(stackItem);
-
-            stackItem = NULL;
-        }
     }
 
     /* Call the standard process utility chain. */
     if (next_ProcessUtility_hook)
-        (*next_ProcessUtility_hook) (parsetree, queryString, context,
-                                     params, dest, completionTag);
+        (*next_ProcessUtility_hook) (pstmt, queryString, context, params,
+                                     queryEnv, dest, completionTag);
     else
-        standard_ProcessUtility(parsetree, queryString, context,
-                                params, dest, completionTag);
+        standard_ProcessUtility(pstmt, queryString, context, params,
+                                queryEnv, dest, completionTag);
 
     /*
      * Process the audit event if there is one.  Also check that this event
@@ -1521,9 +1496,7 @@ pgaudit_ddl_command_end(PG_FUNCTION_ARGS)
         CreateCommandTag(eventData->parsetree);
 
     /* Return objects affected by the (non drop) DDL statement */
-    query = "SELECT pg_catalog.upper(object_type),\n"
-            "       object_identity,\n"
-            "       pg_catalog.upper(command_tag)\n"
+    query = "SELECT UPPER(object_type), object_identity, UPPER(command_tag)\n"
             "  FROM pg_catalog.pg_event_trigger_ddl_commands()";
 
     /* Attempt to connect */
@@ -1624,11 +1597,11 @@ pgaudit_sql_drop(PG_FUNCTION_ARGS)
     contextOld = MemoryContextSwitchTo(contextQuery);
 
     /* Return objects affected by the drop statement */
-    query = "SELECT pg_catalog.upper(object_type),\n"
-            "       object_identity\n"
-            "  FROM pg_catalog.pg_event_trigger_dropped_objects()\n"
-            " WHERE pg_catalog.lower(object_type) operator(pg_catalog.<>) 'type'\n"
-            "   AND schema_name operator(pg_catalog.<>) 'pg_toast'";
+    query = "SELECT UPPER(object_type),\n"
+        "       object_identity\n"
+        "  FROM pg_catalog.pg_event_trigger_dropped_objects()\n"
+        " WHERE lower(object_type) <> 'type'\n"
+        "   AND schema_name <> 'pg_toast'";
 
     /* Attempt to connect */
     result = SPI_connect();
@@ -1846,7 +1819,7 @@ _PG_init(void)
     if (inited)
         return;
 
-    /* Must be loaded with shared_preload_libraries */
+    /* Must be loaded with shared_preload_libaries */
     if (!process_shared_preload_libraries_in_progress)
         ereport(ERROR, (errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
                 errmsg("pgaudit must be loaded via shared_preload_libraries")));
