@@ -643,7 +643,7 @@ ALTER AGGREGATE public.sum_test(integer) RENAME TO sum_test2;
 
 --
 -- Test conversion
-CREATE CONVERSION public.conversion_test FOR 'latin1' TO 'utf8' FROM pg_catalog.iso8859_1_to_utf8;
+CREATE CONVERSION public.conversion_test FOR 'SQL_ASCII' TO 'MULE_INTERNAL' FROM pg_catalog.ascii_to_mic;
 ALTER CONVERSION public.conversion_test RENAME TO conversion_test2;
 
 --
@@ -666,24 +666,6 @@ CREATE SCHEMA foo2
 drop table public.t;
 drop role alice;
 
---
--- Test for non-empty stack error
-CREATE OR REPLACE FUNCTION get_test_id(_ret REFCURSOR) RETURNS REFCURSOR
-LANGUAGE plpgsql IMMUTABLE AS $$
-BEGIN
-    OPEN _ret FOR SELECT 200;
-    RETURN _ret;
-END $$;
-
-BEGIN;
-    SELECT get_test_id('_ret');
-    SELECT get_test_id('_ret2');
-    FETCH ALL FROM _ret;
-    FETCH ALL FROM _ret2;
-    CLOSE _ret;
-    CLOSE _ret2;
-END;
-
 --
 -- Test that frees a memory context earlier than expected
 SET pgaudit.log = 'ALL';
@@ -791,38 +773,6 @@ CREATE TABLE tmp2 AS (SELECT * FROM tmp);
 DROP TABLE tmp;
 DROP TABLE tmp2;
 
---
--- Test that pgaudit event triggers are immune to search-path-based attacks
-
--- Attempt to capture unqualified references to standard functions
-CREATE FUNCTION upper(text) RETURNS text
-LANGUAGE SQL AS 'SELECT (1/0)::text';
-
-CREATE FUNCTION lower(text) RETURNS text
-LANGUAGE SQL AS 'SELECT (1/0)::text';
-
-CREATE FUNCTION my_ne(text, text) RETURNS bool
-LANGUAGE SQL AS 'SELECT (1/0)::bool';
-
-CREATE OPERATOR <> (FUNCTION = my_ne, LEFTARG = text, RIGHTARG = text);
-
-CREATE EXTENSION IF NOT EXISTS pgaudit;
-SET pgaudit.log = 'DDL';
--- Put public schema before pg_catalog to capture unqualified references
-SET search_path = public, pg_catalog;
-
--- If there was a vulnerability, these would fail with division by zero error
-CREATE TABLE wombat ();
-DROP TABLE wombat;
-
-SET pgaudit.log = 'NONE';
-DROP EXTENSION pgaudit;
-
-DROP OPERATOR <> (text, text);
-DROP FUNCTION my_ne(text, text);
-DROP FUNCTION lower(text);
-DROP FUNCTION upper(text);
-
 -- Cleanup
 -- Set client_min_messages up to warning to avoid noise
 SET client_min_messages = 'warning';
